#!/usr/bin/env python
# Copyright (c) Alexey Zasimov <zasimov@gmail.com>
# Copyright (c) Eugene Frolov <eugene@frolov.net.ru>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import time

import yretry
from requests import exceptions
from requests import sessions

from bazooka import exceptions as exc


def retry_on_network_failure(error):
    """Return True on retriable error"""
    return (
        isinstance(error, exc.BaseHTTPException)
        and error.code in yretry.network.RETRY_HTTP_CODES
    ) or yretry.network.is_network_failure(error)


class ReliableSession(sessions.Session):

    def __init__(self, *args, **kwargs):
        super(ReliableSession, self).__init__(*args, **kwargs)
        # By default request time profiling is disabled.
        self._log_duration = False

    @property
    def log_duration(self):
        return self._log_duration

    @log_duration.setter
    def log_duration(self, flag):
        self._log_duration = flag

    def _log_response(self, response, request_time=None):
        """Default handler for response logging.

        request_time(float) contains time of request (in seconds) execution or
                            None if request time profiling is disabled.
        """
        pass

    @yretry.network.retry(retry_on=retry_on_network_failure)
    def request(
        self,
        method,
        url,
        params=None,
        data=None,
        headers=None,
        cookies=None,
        files=None,
        auth=None,
        timeout=None,
        allow_redirects=True,
        proxies=None,
        hooks=None,
        stream=None,
        verify=None,
        cert=None,
        json=None,
    ):
        """See documentation for requests.sessions.Session.request

        Raises exception if status code isn't 2xx
        """
        # Save flag in local stack (user can change this flag using
        # set_profile_request_time property.
        log_duration = self.log_duration

        start_time = None
        request_time = None

        if log_duration:
            start_time = time.time()

        response = super(ReliableSession, self).request(
            method=method,
            url=url,
            params=params,
            data=data,
            headers=headers,
            cookies=cookies,
            files=files,
            auth=auth,
            timeout=timeout,
            allow_redirects=allow_redirects,
            proxies=proxies,
            hooks=hooks,
            stream=stream,
            verify=verify,
            cert=cert,
            json=json,
        )

        if log_duration:
            request_time = time.time() - start_time

        self._log_response(response, request_time)
        try:
            response.raise_for_status()
        except exceptions.HTTPError as e:
            exc.wrap_to_bazooka_exception(e)
        return response



#!/usr/bin/env python
# Copyright (c) Alexey Zasimov <zasimov@gmail.com>
# Copyright (c) Eugene Frolov <eugene@frolov.net.ru>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from bazooka.tests import base

from bazooka import correlation


class CorrelationLoggerAdapterTestCase(base.IsolatedClassTestCase):

    def setUp(self):
        super(CorrelationLoggerAdapterTestCase, self).setUp(
            mock.Mock, correlation.CorrelationLoggerAdapter
        )
        logger = mock.Mock()
        self.correlation_id = "fake correlation id"
        self.adapter = correlation.CorrelationLoggerAdapter(
            logger, self.correlation_id
        )

    def test_process(self):
        """Process returns mmessage with correlation ID."""
        msg = "fake message"
        kwargs = mock.Mock()

        res_msg, res_kwargs = self.adapter.process(msg, kwargs)

        self.assertEqual(res_kwargs, kwargs)
        self.assertEqual(
            res_msg, "[correlation_id=fake correlation id] fake message"
        )


class CorrelationLoggerMixinTestCase(base.TestCase):

    def setUp(self):
        super(CorrelationLoggerMixinTestCase, self).setUp()

        class TestedClass(correlation.CorrelationLoggerMixin):

            def __init__(self):
                self._logger = mock.Mock()
                self.correlation_id = mock.Mock()

        self.mixin = TestedClass()

    @mock.patch("bazooka.correlation.CorrelationLoggerAdapter")
    def test_get_logger_returns_adapter(self, CorrelationLoggerAdapter):
        """CorrelationLoggerMixin.get_logger returns adapter"""
        adapter = mock.Mock()

        CorrelationLoggerAdapter.return_value = adapter

        self.assertEqual(self.mixin.get_logger(), adapter)

    @mock.patch("bazooka.correlation.CorrelationLoggerAdapter")
    def test_get_logger_integration_with_adapter(
        self, CorrelationLoggerAdapter
    ):
        """Check integration with CorrelationLoggerAdapter"""
        self.mixin.get_logger()

        CorrelationLoggerAdapter.assert_called_once_with(
            self.mixin._logger, self.mixin.correlation_id
        )



#!/usr/bin/env python
# Copyright (c) Alexey Zasimov <zasimov@gmail.com>
# Copyright (c) Eugene Frolov <eugene@frolov.net.ru>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from requests import auth as req_auth

from bazooka import correlation
from bazooka import curl_logging
from bazooka import sessions

DEFAULT_TIMEOUT = 300


class MicroserviceSession(
    correlation.CorrelationLoggerMixin,
    curl_logging.CurlLoggingMixin,
    sessions.ReliableSession,
):

    def __init__(
        self,
        auth,
        verify_ssl,
        correlation_id=None,
        correlation_id_header_name="correlationid",
        log_duration=True,
    ):
        super(MicroserviceSession, self).__init__()
        self.log_duration = log_duration
        self.auth = auth
        # Rewrite default requests.session.Session 'verify' value.
        self.verify = verify_ssl
        self._correlation_id = correlation_id
        self.headers[correlation_id_header_name] = self.correlation_id
        self._logger = logging.getLogger(__name__)

    @property
    def correlation_id(self):
        return self._correlation_id

    def _log_response(self, response, request_time=None):
        """Write to log HTTP status code."""
        logger = self.get_logger()
        if request_time is None:
            logger.info(
                "Response: from %s with status code %s %s: %s",
                response.url,
                response.status_code,
                response.reason,
                response.text,
            )
        else:
            logger.info(
                "Response(%s s): from %s with status code %s %s: %s",
                request_time,
                response.url,
                response.status_code,
                response.reason,
                response.text,
            )


class SensitiveMicroserviceSession(
    curl_logging.SensitiveCurlLoggingMixin, MicroserviceSession
):
    pass


class Client(object):
    """Warning! Client raises exceptions

    Standard behaviour of requests client - returns response object.

    Current client raises appropriate exception if response status
    code isn't 2xx

    Client uses 'retries' library to make retries.

    To intialize client you need to pass request auth object or None. See
    documentation for requests.auth.

    Example:
        from requests import auth
        from bazooka import client

        requests = client.BaseClient(
            auth.HTTPBasicAuth('username', 'password'))
        print requests.get('http://localhost:5003').json()

    Request time profiling is enabled by default.
    """

    SESSION = MicroserviceSession

    def __init__(
        self,
        auth=None,
        verify_ssl=True,
        allow_redirects=True,
        correlation_id=None,
        correlation_id_header_name="correlationid",
        log_duration=True,
        default_timeout=DEFAULT_TIMEOUT,
        session=None,
    ):
        super(Client, self).__init__()
        self._auth = auth
        self._verify_ssl = verify_ssl
        self._allow_redirects = allow_redirects
        self._correlation_id = correlation_id
        self._correlation_id_header_name = correlation_id_header_name
        self._log_duration = log_duration
        self._default_timeout = default_timeout
        if session:
            self.SESSION = session

    @property
    def correlation_id(self):
        return self._correlation_id

    @correlation_id.setter
    def correlation_id(self, val):
        self._correlation_id = val

    @property
    def log_duration(self):
        return self._log_duration

    @log_duration.setter
    def log_duration(self, log_duration):
        self._log_duration = log_duration

    def request(self, method, url, **kwargs):
        """See documentation for requests.api.request."""
        with self.SESSION(
            self._auth,
            self._verify_ssl,
            self._correlation_id,
            self._correlation_id_header_name,
            self._log_duration,
        ) as session:
            kwargs.setdefault("timeout", self._default_timeout)
            return session.request(method=method, url=url, **kwargs)

    def get(self, url, params=None, **kwargs):
        """See documentation for requests.api.get."""
        kwargs.setdefault("allow_redirects", self._allow_redirects)
        return self.request("get", url, params=params, **kwargs)

    def options(self, url, **kwargs):
        """See documentation for requests.api.options."""
        kwargs.setdefault("allow_redirects", self._allow_redirects)
        return self.request("options", url, **kwargs)

    def head(self, url, **kwargs):
        """See documentation for requests.api.head."""
        kwargs.setdefault("allow_redirects", False)
        return self.request("head", url, **kwargs)

    def post(self, url, data=None, json=None, **kwargs):
        """See documentation for requests.api.post."""
        return self.request("post", url, data=data, json=json, **kwargs)

    def put(self, url, data=None, **kwargs):
        """See documentation for requests.api.put."""
        return self.request("put", url, data=data, **kwargs)

    def patch(self, url, data=None, **kwargs):
        """See documentation for requests.api.patch."""
        return self.request("patch", url, data=data, **kwargs)

    def delete(self, url, **kwargs):
        """See documentation for requests.api.delete."""
        return self.request("delete", url, **kwargs)


class BasicAuthClient(Client):
    """HTTP client use basic auth token to authentication

    To intialize client you need to pass a username and a password

    Example:
        from bazooka import BasicAuthClient

        requests = client.BasicAuthClient('efrolov', 'mypassword')
        print requests.get('http://localhost:5003').json()

    """

    def __init__(
        self,
        username,
        password,
        verify_ssl=True,
        allow_redirects=True,
        correlation_id=None,
        correlation_id_header_name="correlationid",
        log_duration=True,
        default_timeout=DEFAULT_TIMEOUT,
    ):
        super(BasicAuthClient, self).__init__(
            req_auth.HTTPBasicAuth(username=username, password=password),
            verify_ssl=verify_ssl,
            allow_redirects=allow_redirects,
            correlation_id=correlation_id,
            correlation_id_header_name=correlation_id_header_name,
            log_duration=log_duration,
            default_timeout=default_timeout,
        )



#!/usr/bin/env python
# Copyright (c) Alexey Zasimov <zasimov@gmail.com>
# Copyright (c) Eugene Frolov <eugene@frolov.net.ru>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock
from six import moves

from bazooka.tests import base


def fake_retry(*args, **kwargs):
    def wrapper(f):
        return f

    return wrapper


@mock.patch("yretry.network.retry", fake_retry)
def import_sessions():
    """Loads module for testing with mocked decorator."""
    import bazooka.sessions

    moves.reload_module(bazooka.sessions)
    return bazooka.sessions


sessions = import_sessions()


class ReliableSessionTestCase(base.IsolatedClassTestCase):

    def setUp(self):
        class BaseSession(mock.MagicMock):

            def request(self, *args, **kwargs):
                pass

        self.BaseSession = BaseSession

        super(ReliableSessionTestCase, self).setUp(
            BaseSession, sessions.ReliableSession
        )

        self.session = sessions.ReliableSession()

    def test_log_duration_is_disabled_by_default(self):
        self.assertFalse(self.session.log_duration)

    def test_request_pass_throught(self):
        """Check that request method pass throught arguments."""
        method = mock.Mock()
        url = mock.Mock()
        params = mock.Mock()
        data = mock.Mock()
        headers = mock.Mock()
        cookies = mock.Mock()
        files = mock.Mock()
        auth = mock.Mock()
        timeout = mock.Mock()
        allow_redirects = mock.Mock()
        proxies = mock.Mock()
        hooks = mock.Mock()
        stream = mock.Mock()
        verify = mock.Mock()
        cert = mock.Mock()
        json = mock.Mock()

        response = mock.MagicMock()

        with mock.patch.object(
            self.BaseSession, "request", return_value=response
        ):
            self.BaseSession.request.return_value = response

            self.assertEqual(
                self.session.request(
                    method,
                    url,
                    params,
                    data,
                    headers,
                    cookies,
                    files,
                    auth,
                    timeout,
                    allow_redirects,
                    proxies,
                    hooks,
                    stream,
                    verify,
                    cert,
                    json,
                ),
                response,
            )

            self.BaseSession.request.assert_called_once_with(
                method=method,
                url=url,
                params=params,
                data=data,
                headers=headers,
                cookies=cookies,
                files=files,
                auth=auth,
                timeout=timeout,
                allow_redirects=allow_redirects,
                proxies=proxies,
                hooks=hooks,
                stream=stream,
                verify=verify,
                cert=cert,
                json=json,
            )

    def test_request_calls_raise_for_status(self):
        """Check that request method calls raise_for_status."""
        method = mock.Mock()
        url = mock.Mock()

        response = mock.MagicMock()

        with mock.patch.object(
            self.BaseSession, "request", return_value=response
        ):
            self.BaseSession.request.return_value = response

            self.session.request(method, url)

            response.raise_for_status.assert_called_once_with()

    def test_request_calls_log_response_without_request_time(self):
        """Check that request method calls _log_response.

        Request time profiling is disabled by default.

        request_time is None in this case.
        """
        method = mock.Mock()
        url = mock.Mock()

        response = mock.MagicMock()

        with mock.patch.object(
            self.BaseSession, "request", return_value=response
        ):
            with mock.patch.object(
                self.session, "_log_response"
            ) as log_response:
                self.BaseSession.request.return_value = response

                self.session.request(method, url)

                log_response.assert_called_once_with(response, None)

    def test_log_duration_setter(self):
        """[Positive] I can change log_duration flag."""
        current = self.session._log_duration
        self.session.log_duration = not current
        self.assertEqual(self.session._log_duration, not current)

    @mock.patch("time.time")
    def test_request_calls_log_response_with_request_time(self, time):
        """Check that request method calls _log_response.

        Request time profiling is enabled.

        """
        method = mock.Mock()
        url = mock.Mock()

        response = mock.MagicMock()

        time.side_effect = [1, 2]

        self.session.log_duration = True

        with mock.patch.object(
            self.BaseSession, "request", return_value=response
        ):
            with mock.patch.object(
                self.session, "_log_response"
            ) as log_response:
                self.BaseSession.request.return_value = response

                self.session.request(method, url)

                log_response.assert_called_once_with(response, 1)



#!/usr/bin/env python
# Copyright (c) Alexey Zasimov <zasimov@gmail.com>
# Copyright (c) Eugene Frolov <eugene@frolov.net.ru>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

if sys.version_info[:2] >= (2, 7):
    import unittest
else:
    import unittest2 as unittest

import mock


class TestCase(unittest.TestCase):
    pass


class IsolatedClassTestCase(TestCase):
    """Test class with fake base class."""

    def setUp(self, base_class, tested_class):
        super(IsolatedClassTestCase, self).setUp()
        self.base_class_patcher = mock.patch.object(
            tested_class, "__bases__", (base_class,)
        )
        self.base_class_patcher.start()
        self.base_class_patcher.is_local = True

    def tearDown(self):
        self.base_class_patcher.stop()
        super(IsolatedClassTestCase, self).tearDown()



#!/usr/bin/env python
# Copyright (c) Alexey Zasimov <zasimov@gmail.com>
# Copyright (c) Eugene Frolov <eugene@frolov.net.ru>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from bazooka import sessions


def request(method, url, **kwargs):
    """See documentation for requests.api.request."""
    with sessions.ReliableSession() as session:
        return session.request(method=method, url=url, **kwargs)


def get(url, params=None, **kwargs):
    """See documentation for requests.api.get."""
    kwargs.setdefault("allow_redirects", True)
    return request("get", url, params=params, **kwargs)


def options(url, **kwargs):
    """See documentation for requests.api.options."""
    kwargs.setdefault("allow_redirects", True)
    return request("options", url, **kwargs)


def head(url, **kwargs):
    """See documentation for requests.api.head."""
    kwargs.setdefault("allow_redirects", False)
    return request("head", url, **kwargs)


def post(url, data=None, json=None, **kwargs):
    """See documentation for requests.api.post."""
    return request("post", url, data=data, json=json, **kwargs)


def put(url, data=None, **kwargs):
    """See documentation for requests.api.put."""
    return request("put", url, data=data, **kwargs)


def patch(url, data=None, **kwargs):
    """See documentation for requests.api.patch."""
    return request("patch", url, data=data, **kwargs)


def delete(url, **kwargs):
    """See documentation for requests.api.delete."""
    return request("delete", url, **kwargs)



#!/usr/bin/env python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
# Copyright 2020 Alexandr Popov
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
from six import moves

import bazooka.exceptions
import bazooka.sessions
from bazooka.tests import base


class FakeResponse(object):
    def __init__(self):
        self.call_count = 0

    def raise_for_status(self):
        self.call_count += 1
        raise bazooka.exceptions.BaseHTTPException(
            mock.Mock(response=mock.MagicMock(status_code=500))
        )


class RetryableSessionTestCase(base.TestCase):

    @mock.patch("time.sleep", return_value=None)
    def test_request_get_retry(self, _patched_sleep):
        """Check that request played more than once on error"""
        moves.reload_module(bazooka.sessions)
        session = bazooka.sessions.ReliableSession()
        response = FakeResponse()

        with mock.patch(
            "requests.sessions.Session.request", return_value=response
        ):
            with self.assertRaises(bazooka.exceptions.BaseHTTPException):
                session.request("GET", "")
        self.assertGreater(response.call_count, 1)



#!/usr/bin/env python
# Copyright (c) Alexey Zasimov <zasimov@gmail.com>
# Copyright (c) Eugene Frolov <eugene@frolov.net.ru>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging


class CorrelationLoggerAdapter(logging.LoggerAdapter):
    """It writes correlation id for each log entry"""

    def __init__(self, logger, correlation_id):
        super(CorrelationLoggerAdapter, self).__init__(logger, {})
        self._correlation_id = correlation_id

    def process(self, msg, kwargs):
        return "[correlation_id=%s] %s" % (self._correlation_id, msg), kwargs


class CorrelationLoggerMixin(object):
    """Adds ability to write correlation id to log

    Basic class must provider self.correlation_id property that
    contains current correlation identifier.

    Mixin overloads `get_logger` method that returns
    CorrelationLoggerAdapter. `get_logger` method is it special
    method that used by retries module.
    """

    def get_logger(self):
        adapter = CorrelationLoggerAdapter(self._logger, self.correlation_id)
        return adapter



#!/usr/bin/env python
# Copyright (c) Alexey Zasimov <zasimov@gmail.com>
# Copyright (c) Eugene Frolov <eugene@frolov.net.ru>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock
from six import moves

from bazooka.tests import base

from bazooka import client
from bazooka import sessions


class MicroserviceSessionInitializationTestCase(base.TestCase):

    def test_verify_ssl_flag(self):
        """[Positive] MicroserviceSession rewrites verify flag."""
        auth = mock.Mock()
        verify_ssl = mock.Mock()
        # XXX(Alexey Zasimov): Restore ReliableSession after mock.
        #                      IsolatedClassTestCases doesn't work correctly.
        moves.reload_module(client)
        session = client.MicroserviceSession(auth=auth, verify_ssl=verify_ssl)
        self.assertEqual(session.verify, verify_ssl)

    def test_correlation_id_in_headers(self):
        """[Positive] Correlation id  in headers."""
        auth = mock.Mock()
        verify_ssl = mock.Mock()
        correlation_id = mock.Mock()
        correlation_id_header_name = mock.Mock()
        # XXX(Alexey Zasimov): Restore ReliableSession after mock.
        #                      IsolatedClassTestCases doesn't work correctly.
        moves.reload_module(client)
        session = client.MicroserviceSession(
            auth=auth,
            verify_ssl=verify_ssl,
            correlation_id=correlation_id,
            correlation_id_header_name=correlation_id_header_name,
        )

        self.assertEqual(
            session.headers[correlation_id_header_name], correlation_id
        )

    def test_log_duration_is_set(self):
        """[Positive] I can set log duration flag for MicroserviceSession."""
        auth = mock.Mock()
        verify_ssl = mock.Mock()
        correlation_id = mock.Mock()
        correlation_id_header_name = mock.Mock()

        # XXX(Alexey Zasimov): Restore ReliableSession after mock.
        #                      IsolatedClassTestCases doesn't work correctly.
        moves.reload_module(client)

        session = client.MicroserviceSession(
            auth=auth,
            verify_ssl=verify_ssl,
            correlation_id=correlation_id,
            correlation_id_header_name=correlation_id_header_name,
            log_duration=True,
        )

        self.assertTrue(session.log_duration)


class SensitiveMicroserviceSessionInitializationTestCase(base.TestCase):

    def test_curl_mixin_override(self):
        auth = mock.Mock()
        verify_ssl = mock.Mock()
        correlation_id = mock.Mock()
        correlation_id_header_name = mock.Mock()
        # XXX(Alexey Zasimov): Restore ReliableSession after mock.
        #                      IsolatedClassTestCases doesn't work correctly.
        moves.reload_module(client)
        session = client.SensitiveMicroserviceSession(
            auth=auth,
            verify_ssl=verify_ssl,
            correlation_id=correlation_id,
            correlation_id_header_name=correlation_id_header_name,
        )

        self.assertEqual(session._sanitize_body("123"), session.SANITIZED_PLUG)


class MicroserviceSessionTestCase(base.TestCase):

    def setUp(self):
        super(MicroserviceSessionTestCase, self).setUp()
        auth = mock.Mock()
        verify_ssl = mock.Mock()
        self.session = client.MicroserviceSession(
            auth=auth, verify_ssl=verify_ssl
        )

    def test_log_response_without_request_time(self):
        """[Positive] Logging for response works."""
        logger = mock.MagicMock()

        response = mock.MagicMock()
        response.url = "fake url"
        response.status_code = "fake status code"
        response.reason = "fake reason"
        response.text = "fake response"

        with mock.patch.object(
            self.session, "get_logger", return_value=logger
        ):
            self.assertIsNone(self.session._log_response(response))

            logger.info.assert_called_once_with(
                "Response: from %s with status code %s %s: %s",
                response.url,
                response.status_code,
                response.reason,
                response.text,
            )

    def test_log_response_with_request_time(self):
        """[Positive] Logging for response works (+ request time)."""
        logger = mock.MagicMock()

        request_time = 79.87

        response = mock.MagicMock()
        response.url = "fake url"
        response.status_code = "fake status code"
        response.reason = "fake reason"
        response.text = "fake response"

        with mock.patch.object(
            self.session, "get_logger", return_value=logger
        ):
            self.assertIsNone(
                self.session._log_response(response, request_time)
            )

            logger.info.assert_called_once_with(
                "Response(%s s): from %s with status code %s %s: %s",
                request_time,
                response.url,
                response.status_code,
                response.reason,
                response.text,
            )


class ClientTestCase(base.TestCase):

    def setUp(self):
        super(ClientTestCase, self).setUp()

        self.basic_auth_token = mock.Mock()
        self.auth_user_token = mock.Mock()

        self.auth = mock.Mock()

        self.verify_ssl = mock.Mock()
        self.allow_redirects = mock.Mock()
        self.correlation_id = mock.Mock()
        self.correlation_id_header_name = mock.Mock()
        self.log_duration = mock.Mock()

        self.client = client.Client(
            auth=self.auth,
            verify_ssl=self.verify_ssl,
            allow_redirects=self.allow_redirects,
            correlation_id=self.correlation_id,
            correlation_id_header_name=self.correlation_id_header_name,
            log_duration=self.log_duration,
        )

    def test_session_initialization(self):
        """[Positive] session is initialized with valid parameters."""

        with mock.patch.object(self.client, "SESSION"):
            self.client.request("FAKEMETHOD", "FAKE_URL")

            self.client.SESSION.assert_called_once_with(
                self.auth,
                self.verify_ssl,
                self.correlation_id,
                self.correlation_id_header_name,
                self.log_duration,
            )

    def test_get_calls_request(self):
        """get method calls request with valid arguments."""

        url = mock.Mock()
        params = mock.Mock()
        kwargs = {"fake": 1}

        response = mock.Mock()

        with mock.patch.object(self.client, "request") as request:
            request.return_value = response

            self.assertEqual(self.client.get(url, params, **kwargs), response)

            request.assert_called_once_with(
                "get",
                url,
                params=params,
                allow_redirects=self.allow_redirects,
                fake=1,
            )

    def test_options_calls_request(self):
        """options method calls request with valid arguments."""

        url = mock.Mock()
        kwargs = {"fake": 1}

        response = mock.Mock()

        with mock.patch.object(self.client, "request") as request:
            request.return_value = response

            self.assertEqual(self.client.options(url, **kwargs), response)

            request.assert_called_once_with(
                "options", url, allow_redirects=self.allow_redirects, fake=1
            )

    def test_head_calls_request(self):
        """head method calls request with valid arguments."""

        url = mock.Mock()
        kwargs = {"fake": 1}

        response = mock.Mock()

        with mock.patch.object(self.client, "request") as request:
            request.return_value = response

            self.assertEqual(self.client.head(url, **kwargs), response)

            request.assert_called_once_with(
                "head", url, allow_redirects=False, fake=1
            )

    def test_post_calls_request(self):
        """get method calls request with valid arguments."""

        url = mock.Mock()
        data = mock.Mock()
        json = mock.Mock()
        kwargs = {"fake": 1}

        response = mock.Mock()

        with mock.patch.object(self.client, "request") as request:
            request.return_value = response

            self.assertEqual(
                self.client.post(url, data, json, **kwargs), response
            )

            request.assert_called_once_with(
                "post", url, data=data, json=json, fake=1
            )

    def test_put_calls_request(self):
        """get method calls request with valid arguments."""

        url = mock.Mock()
        data = mock.Mock()
        kwargs = {"fake": 1}

        response = mock.Mock()

        with mock.patch.object(self.client, "request") as request:
            request.return_value = response

            self.assertEqual(self.client.put(url, data, **kwargs), response)

            request.assert_called_once_with("put", url, data=data, fake=1)

    def test_delete_calls_request(self):
        """delete method calls request with valid arguments."""

        url = mock.Mock()
        kwargs = {"fake": 1}

        response = mock.Mock()

        with mock.patch.object(self.client, "request") as request:
            request.return_value = response

            self.assertEqual(self.client.delete(url, **kwargs), response)

            request.assert_called_once_with("delete", url, fake=1)

    def test_request_with_default_timeout(self):
        """request method calls with valid defsault timeout."""

        url = mock.Mock()
        timeouts = [45, 60, None]

        moves.reload_module(client)
        for timeout in timeouts:
            kwargs = {"default_timeout": timeout} if timeout else {}
            http_client = client.Client(**kwargs)

            session_cls = sessions.ReliableSession
            with mock.patch.object(session_cls, "request") as request:
                http_client.request("get", url)
                expected = timeout or client.DEFAULT_TIMEOUT
                request.assert_called_once_with(
                    method="get", url=url, timeout=expected
                )


class MicroserviceClientTestCase(base.TestCase):

    def test_client_instantiation(self):
        """[Positive] is super called correct (Client class)."""

        auth = mock.Mock()

        cli = client.Client(
            auth=auth,
            verify_ssl="verify_ssl",
            allow_redirects="allow_redirects",
            correlation_id="correlation_id",
            correlation_id_header_name="correlation_id_header_name",
            log_duration="log_duration",
        )

        self.assertIs(cli._auth, auth)
        self.assertIs(cli._verify_ssl, "verify_ssl")
        self.assertIs(cli._allow_redirects, "allow_redirects")
        self.assertIs(cli._correlation_id, "correlation_id")
        self.assertIs(
            cli._correlation_id_header_name, "correlation_id_header_name"
        )
        self.assertIs(cli._log_duration, "log_duration")


class SensitiveMicroserviceClientTestCase(base.TestCase):

    def test_client_instantiation(self):
        """[Positive] is super called correct (Client class)."""

        auth = mock.Mock()

        cli = client.Client(
            auth=auth,
            verify_ssl="verify_ssl",
            allow_redirects="allow_redirects",
            correlation_id="correlation_id",
            correlation_id_header_name="correlation_id_header_name",
            log_duration="log_duration",
            session=client.SensitiveMicroserviceSession,
        )

        self.assertIs(cli._auth, auth)
        self.assertIs(cli._verify_ssl, "verify_ssl")
        self.assertIs(cli._allow_redirects, "allow_redirects")
        self.assertIs(cli._correlation_id, "correlation_id")
        self.assertIs(
            cli._correlation_id_header_name, "correlation_id_header_name"
        )
        self.assertIs(cli._log_duration, "log_duration")
        self.assertIs(cli.SESSION, client.SensitiveMicroserviceSession)


class BasicAuthClientTestCase(base.TestCase):

    @mock.patch("requests.auth.HTTPBasicAuth")
    def test_client_instantiation(self, auth_mock):
        """[Positive] is super called correct (BasicAuthClient class)."""

        cli = client.BasicAuthClient(
            username="username",
            password="password",
            verify_ssl="verify_ssl",
            allow_redirects="allow_redirects",
            correlation_id="correlation_id",
            correlation_id_header_name="correlation_id_header_name",
            log_duration="log_duration",
        )

        auth_mock.assert_called_once_with(
            username="username", password="password"
        )
        self.assertIs(cli._auth, auth_mock())
        self.assertIs(cli._verify_ssl, "verify_ssl")
        self.assertIs(cli._allow_redirects, "allow_redirects")
        self.assertIs(cli._correlation_id, "correlation_id")
        self.assertIs(
            cli._correlation_id_header_name, "correlation_id_header_name"
        )
        self.assertIs(cli._log_duration, "log_duration")



#!/usr/bin/env python
# Copyright (c) Alexey Zasimov <zasimov@gmail.com>
# Copyright (c) Eugene Frolov <eugene@frolov.net.ru>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections

import mock

from bazooka.tests import base

from bazooka import curl_logging


class BaseMixinTestCase(base.TestCase):
    mixin_class = None

    def setUp(self):
        """BaseMixinTestCase

        Declare TestedClass with CurlLoggingMixin.
        Set SuperClass as a parent for TestClass.
        """

        class SuperClass(object):

            def prepare_request(self, request):
                pass

            def get_logger(self):
                pass

        class TestedClass(self.mixin_class, SuperClass):
            pass

        self.SuperClass = SuperClass
        self.TestedClass = TestedClass

        super(BaseMixinTestCase, self).setUp()

        self.mixin = TestedClass()


class CurlLoggingMixinTestCase(BaseMixinTestCase):
    mixin_class = curl_logging.CurlLoggingMixin

    def test_prepare_request_calls_log_request(self):
        """prepare_requests invokes super method and _log_request."""
        request = mock.MagicMock()

        with mock.patch.object(
            self.SuperClass, "prepare_request"
        ) as prepare_request:
            with mock.patch.object(self.mixin, "_log_request") as log_request:
                prepare_request.return_value = request

                self.assertEqual(self.mixin.prepare_request(request), request)

                prepare_request.assert_called_once_with(request)

                log_request.assert_called_once_with(request)

    def test_hide_sensitive_headers_is_dict(self):
        """_hide_sensitive_headers hide value for private1 and private2."""
        data = {"public1": 1, "public2": 2, "private1": "a", "private2": "b"}

        with mock.patch.object(
            self.mixin, "SENSITIVE_HEADERS", ("PRIVATE1", "PRIVATE2")
        ):
            filtered = self.mixin._hide_sensitive_headers(data)

            self.assertDictEqual(
                filtered,
                {
                    "public1": 1,
                    "public2": 2,
                    "private1": "<private1>",
                    "private2": "<private2>",
                },
            )

    def test_curlify_request(self):
        """CURLification is working."""
        request = mock.MagicMock()
        request.headers = mock.MagicMock()
        request.body = "body"
        request.method = "GET"
        request.url = "http://super"

        headers = collections.OrderedDict([("H1", "1"), ("H2", "2")])

        with mock.patch.object(
            self.mixin, "_hide_sensitive_headers", return_value=headers
        ):
            self.assertEqual(
                self.mixin._curlify_request(request),
                "curl -X 'GET' -H 'H1: 1' -H 'H2: 2' -d 'body' http://super",
            )

    def test_hide_sensitive_headers(self):
        fake_value = "fake_value"

        seq = curl_logging.CurlLoggingMixin.SENSITIVE_HEADERS

        secret_payload = {k: fake_value for k in seq}
        cleaned_secret_payload = self.mixin._hide_sensitive_headers(
            secret_payload
        )

        non_secret_payload = {"%s_non_secret" % k: fake_value for k in seq}
        cleaned_non_secret_payload = self.mixin._hide_sensitive_headers(
            non_secret_payload
        )

        for k, v in cleaned_secret_payload.items():
            self.assertEqual(self.mixin._mask(k), v)

        for v in cleaned_non_secret_payload.values():
            self.assertEqual(v, fake_value)

    def test_hide_sensitive_headers_mask_value(self):
        value = "abc"
        expected_value = "<%s>" % value

        self.assertEqual(self.mixin._mask(value), expected_value)

    def test_log_request_sequence(self):
        """_log_request sequence is valid."""
        request = mock.Mock()

        logger = mock.MagicMock()
        curl_cmd = "curlification"

        with mock.patch.object(
            self.mixin, "get_logger", return_value=logger
        ) as get_logger:

            with mock.patch.object(
                self.mixin, "_curlify_request", return_value=curl_cmd
            ) as curlify_request:

                self.mixin._log_request(request)

                get_logger.assert_called_once_with()
                curlify_request.assert_called_once_with(request)

                logger.info.assert_called_once_with(
                    "HTTP(s) request: %s", curl_cmd
                )


class SensitiveCurlLoggingMixinTestCase(BaseMixinTestCase):
    mixin_class = curl_logging.SensitiveCurlLoggingMixin

    def test_curlify_request(self):
        request = mock.MagicMock()
        request.headers = mock.MagicMock()
        request.body = "very_important_body"
        request.method = "POST"
        request.url = "http://super"

        headers = collections.OrderedDict([("H1", "1"), ("H2", "2")])

        with mock.patch.object(
            self.mixin, "_hide_sensitive_headers", return_value=headers
        ):
            self.assertEqual(
                self.mixin._curlify_request(request),
                "curl -X 'POST' -H 'H1: 1' -H 'H2: 2' -d '%s' "
                "http://super" % self.mixin.SANITIZED_PLUG,
            )



#!/usr/bin/env python
# Copyright (c) Alexey Zasimov <zasimov@gmail.com>
# Copyright (c) Eugene Frolov <eugene@frolov.net.ru>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class CurlLoggingMixin(object):
    """Mixin adds request logging in curl format (for each request)

    Formatted request will be logged with INFO level.

    Values of SENSITIVE_HEADERS will be wiped out from logs.

    Mixin can be used with request.sessions.Session class because it
    overloads prepare_request method to add curl-like logging.
    """

    # Should be uppercase
    SENSITIVE_HEADERS = {
        "TOKEN",
        "AUTHORIZATION",
        "BASICAUTH",
        "X-API-KEY",
        "X-AUTH-TOKEN",
        "X-SERVICE-TOKEN",
        "COOKIE",
    }

    def prepare_request(self, request):
        request = super(CurlLoggingMixin, self).prepare_request(request)
        self._log_request(request)
        return request

    @staticmethod
    def _mask(value):
        return "<%s>" % value

    def _hide_sensitive_headers(self, data):
        if isinstance(data, dict):
            data = data.copy()

            for param in data:
                if str(param).upper() in self.SENSITIVE_HEADERS:
                    data[param] = self._mask(param)

        return data

    def _sanitize_body(self, body):
        return body

    def _curlify_request(self, request):
        """OpenStack approach for human-readable requests logging."""
        parameters = dict()
        headers = self._hide_sensitive_headers(dict(request.headers))

        parameters["headers"] = " ".join(
            ["-H '%s: %s'" % (k, v) for k, v in headers.items()]
        )

        parameters["data"] = (
            ("-d '%s'" % self._sanitize_body(request.body))
            if request.body is not None
            else ""
        )

        parameters["method"] = "-X '%s'" % request.method
        parameters["url"] = request.url

        return "curl %(method)s %(headers)s %(data)s %(url)s" % parameters

    def _log_request(self, request):
        logger = self.get_logger()
        curl_cmd = self._curlify_request(request)
        logger.info("HTTP(s) request: %s", curl_cmd)


class SensitiveCurlLoggingMixin(CurlLoggingMixin):
    SANITIZED_PLUG = "<SENSITIVE_DATA>"

    def _sanitize_body(self, body):
        return self.SANITIZED_PLUG



# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
# Copyright 2018 Mail.ru Group
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from requests import exceptions
from six.moves import http_client as httplib


class BaseHTTPException(Exception):

    def __init__(self, cause):
        super(BaseHTTPException, self).__init__(str(cause))
        self._cause = cause
        self._code = self._cause.response.status_code

    @property
    def cause(self):
        return self._cause

    @property
    def code(self):
        return self._code


class ClientError(BaseHTTPException):
    pass


class NotFoundError(ClientError):
    pass


class ConflictError(ClientError):
    pass


def wrap_to_bazooka_exception(cause):
    if isinstance(cause, exceptions.HTTPError):
        if httplib.NOT_FOUND == cause.response.status_code:
            raise NotFoundError(cause)
        elif httplib.CONFLICT == cause.response.status_code:
            raise ConflictError(cause)
        raise BaseHTTPException(cause)
    raise cause



#!/usr/bin/env python
# Copyright (c) Alexey Zasimov <zasimov@gmail.com>
# Copyright (c) Eugene Frolov <eugene@frolov.net.ru>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from bazooka.api import delete
from bazooka.api import get
from bazooka.api import patch
from bazooka.api import post
from bazooka.api import put
from bazooka.api import request
from bazooka.client import Client


__all__ = ["request", "get", "post", "put", "delete", "patch", "Client"]
